@@ -6,20 +6,26 @@ from typing import AbstractSet, Any, Callable, Dict, Mapping, Optional, Sequence
 
 from dagster_dbt.cli.types import DbtCliOutput
 from dagster_dbt.cli.utils import execute_cli
-from dagster_dbt.utils import generate_materializations
+from dagster_dbt.types import DbtOutput
+from dagster_dbt.utils import generate_events
 
 from dagster import (
     AssetKey,
+    AssetMaterialization,
+    AssetsDefinition,
+    In,
     MetadataValue,
+    Nothing,
     Out,
     Output,
     SolidExecutionContext,
     TableColumn,
     TableSchema,
-    check,
-    get_dagster_logger,
 )
-from dagster.core.asset_defs import AssetsDefinition, multi_asset
+from dagster import _check as check
+from dagster import get_dagster_logger, op
+from dagster.core.definitions.metadata import RawMetadataValue
+from dagster.core.errors import DagsterInvalidSubsetError
 
 
 def _load_manifest_for_project(
@@ -47,12 +53,70 @@ def _load_manifest_for_project(
         return json.load(f), cli_output
 
 
+def _select_unique_ids_from_manifest_json(
+    manifest_json: Mapping[str, Any], select: str
+) -> AbstractSet[str]:
+    """Method to apply a selection string to an existing manifest.json file."""
+    try:
+        import dbt.graph.cli as graph_cli
+        import dbt.graph.selector as graph_selector
+        from dbt.contracts.graph.manifest import Manifest
+        from networkx import DiGraph
+    except ImportError:
+        check.failed(
+            "In order to use the `select` argument on load_assets_from_dbt_manifest, you must have"
+            "`dbt-core >= 1.0.0` and `networkx` installed."
+        )
+
+    class _DictShim(dict):
+        """Shim to enable hydrating a dictionary into a dot-accessible object"""
+
+        def __getattr__(self, item):
+            ret = super().get(item)
+            # allow recursive access e.g. foo.bar.baz
+            return _DictShim(ret) if isinstance(ret, dict) else ret
+
+    # generate a dbt-compatible graph from the existing child map
+    graph = graph_selector.Graph(DiGraph(incoming_graph_data=manifest_json["child_map"]))
+    manifest = Manifest(
+        # dbt expects dataclasses that can be accessed with dot notation, not bare dictionaries
+        nodes={unique_id: _DictShim(info) for unique_id, info in manifest_json["nodes"].items()},
+        sources={
+            unique_id: _DictShim(info) for unique_id, info in manifest_json["sources"].items()
+        },
+    )
+
+    # create a parsed selection from the select string
+    parsed_spec = graph_cli.parse_union([select], True)
+
+    # execute this selection against the graph
+    selector = graph_selector.NodeSelector(graph, manifest)
+    selected, _ = selector.select_nodes(parsed_spec)
+    if len(selected) == 0:
+        raise DagsterInvalidSubsetError(f"No dbt models match the selection string '{select}'.")
+    return selected
+
+
 def _get_node_name(node_info: Mapping[str, Any]):
     return "__".join([node_info["resource_type"], node_info["package_name"], node_info["name"]])
 
 
 def _get_node_asset_key(node_info):
-    return AssetKey(node_info["name"])
+    if node_info.get("schema") is not None:
+        components = [node_info["schema"], node_info["name"]]
+    else:
+        components = [node_info["name"]]
+
+    return AssetKey(components)
+
+
+def _get_node_description(node_info):
+    code_block = textwrap.indent(node_info["raw_sql"], "    ")
+    description_sections = [
+        node_info["description"],
+        f"#### Raw SQL:\n```\n{code_block}\n```",
+    ]
+    return "\n\n".join(filter(None, description_sections))
 
 
 def _dbt_nodes_to_assets(
@@ -60,82 +124,131 @@ def _dbt_nodes_to_assets(
     select: str,
     selected_unique_ids: AbstractSet[str],
     runtime_metadata_fn: Optional[
-        Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, Any]]
+        Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
     ] = None,
     io_manager_key: Optional[str] = None,
     node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
+    use_build_command: bool = False,
 ) -> AssetsDefinition:
+
     outs: Dict[str, Out] = {}
-    sources: Set[AssetKey] = set()
+    asset_ins: Dict[AssetKey, Tuple[str, In]] = {}
+
+    asset_deps: Dict[AssetKey, Set[AssetKey]] = {}
+
     out_name_to_node_info: Dict[str, Mapping[str, Any]] = {}
-    internal_asset_deps: Dict[str, Set[AssetKey]] = {}
+
     package_name = None
     for unique_id in selected_unique_ids:
-        asset_deps = set()
+        cur_asset_deps = set()
         node_info = dbt_nodes[unique_id]
+        if node_info["resource_type"] != "model":
+            continue
         package_name = node_info.get("package_name", package_name)
+
         for dep_name in node_info["depends_on"]["nodes"]:
             dep_type = dbt_nodes[dep_name]["resource_type"]
-            # ignore seeds/snapshots
+            # ignore seeds/snapshots/tests
             if dep_type not in ["source", "model"]:
                 continue
             dep_asset_key = node_info_to_asset_key(dbt_nodes[dep_name])
 
             # if it's a source, it will be used as an input to this multi-asset
             if dep_type == "source":
-                sources.add(dep_asset_key)
-            # regardless of type, list this as a dependency for the current asset
-            asset_deps.add(dep_asset_key)
-        code_block = textwrap.indent(node_info["raw_sql"], "    ")
-        description_sections = [
-            node_info["description"],
-            f"#### Raw SQL:\n```\n{code_block}\n```",
-        ]
-        description = "\n\n".join(filter(None, description_sections))
+                asset_ins[dep_asset_key] = (dep_name.replace(".", "_"), In(Nothing))
 
+            # regardless of type, list this as a dependency for the current asset
+            cur_asset_deps.add(dep_asset_key)
+
+        # generate the Out that corresponds to this model
         node_name = node_info["name"]
         outs[node_name] = Out(
-            dagster_type=None,
-            asset_key=node_info_to_asset_key(node_info),
-            description=description,
+            description=_get_node_description(node_info),
             io_manager_key=io_manager_key,
             metadata=_columns_to_metadata(node_info["columns"]),
+            is_required=False,
         )
         out_name_to_node_info[node_name] = node_info
-        internal_asset_deps[node_name] = asset_deps
+
+        # set the asset dependencies for this asset
+        asset_deps[node_info_to_asset_key(node_info)] = cur_asset_deps
 
     # prevent op name collisions between multiple dbt multi-assets
     op_name = f"run_dbt_{package_name}"
     if select != "*":
         op_name += "_" + hashlib.md5(select.encode()).hexdigest()[-5:]
 
-    @multi_asset(
+    @op(
         name=op_name,
-        non_argument_deps=sources,
-        outs=outs,
+        tags={"kind": "dbt"},
+        ins=dict(asset_ins.values()),
+        out=outs,
         required_resource_keys={"dbt"},
-        compute_kind="dbt",
-        internal_asset_deps=internal_asset_deps,
     )
-    def _dbt_project_multi_assset(context):
-        dbt_output = context.resources.dbt.run(select=select)
-        # yield an Output for each materialization generated in the run
-        for materialization in generate_materializations(dbt_output):
-            output_name = materialization.asset_key.path[-1]
-            if runtime_metadata_fn:
-                yield Output(
-                    value=None,
-                    output_name=output_name,
-                    metadata=runtime_metadata_fn(context, out_name_to_node_info[output_name]),
-                )
+    def dbt_op(context):
+        dbt_output = None
+        try:
+            # in the case that we're running everything, opt for the cleaner selection string
+            if len(context.selected_output_names) == len(outs):
+                subselect = select
             else:
-                yield Output(
-                    value=None,
-                    output_name=output_name,
-                    metadata_entries=materialization.metadata_entries,
-                )
+                # for each output that we want to emit, translate to a dbt select string by converting
+                # the out to it's corresponding fqn
+                subselect = [
+                    ".".join(out_name_to_node_info[out_name]["fqn"])
+                    for out_name in context.selected_output_names
+                ]
 
-    return _dbt_project_multi_assset
+            if use_build_command:
+                dbt_output = context.resources.dbt.build(select=subselect)
+            else:
+                dbt_output = context.resources.dbt.run(select=subselect)
+
+        finally:
+            # in the case that the project only partially runs successfully, still attempt to generate
+            # events for the parts that were successful
+            if dbt_output is None:
+                dbt_output = DbtOutput(result=context.resources.dbt.get_run_results_json())
+
+            # yield an Output for each materialization generated in the run
+            for event in generate_events(
+                dbt_output,
+                node_info_to_asset_key=node_info_to_asset_key,
+                manifest_json=context.resources.dbt.get_manifest_json(),
+            ):
+                # convert AssetMaterializations to outputs
+                if isinstance(event, AssetMaterialization):
+                    output_name = event.asset_key.path[-1]
+                    if runtime_metadata_fn:
+                        yield Output(
+                            value=None,
+                            output_name=output_name,
+                            metadata=runtime_metadata_fn(
+                                context, out_name_to_node_info[output_name]
+                            ),
+                        )
+                    else:
+                        yield Output(
+                            value=None,
+                            output_name=output_name,
+                            metadata_entries=event.metadata_entries,
+                        )
+                # yield AssetObservations normally
+                else:
+                    yield event
+
+    return AssetsDefinition(
+        asset_keys_by_input_name={
+            input_name: asset_key for asset_key, (input_name, _) in asset_ins.items()
+        },
+        asset_keys_by_output_name={
+            output_name: node_info_to_asset_key(out_name_to_node_info[output_name])
+            for output_name in outs.keys()
+        },
+        node_def=dbt_op,
+        can_subset=True,
+        asset_deps=asset_deps,
+    )
 
 
 def _columns_to_metadata(columns: Mapping[str, Any]) -> Optional[Mapping[str, Any]]:
@@ -169,6 +282,7 @@ def load_assets_from_dbt_project(
     ] = None,
     io_manager_key: Optional[str] = None,
     node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
+    use_build_command: bool = False,
 ) -> Sequence[AssetsDefinition]:
     """
     Loads a set of DBT models from a DBT project into Dagster assets.
@@ -193,6 +307,9 @@ def load_assets_from_dbt_project(
         node_info_to_asset_key: (Mapping[str, Any] -> AssetKey): A function that takes a dictionary
             of dbt node info and returns the AssetKey that you want to represent that node. By
             default, the asset key will simply be the name of the dbt model.
+        use_build_command: (bool): Flag indicating if you want to use `dbt build` as the core computation
+            for this asset, rather than `dbt run`.
+
     """
     check.str_param(project_dir, "project_dir")
     profiles_dir = check.opt_str_param(
@@ -216,6 +333,7 @@ def load_assets_from_dbt_project(
             runtime_metadata_fn=runtime_metadata_fn,
             io_manager_key=io_manager_key,
             node_info_to_asset_key=node_info_to_asset_key,
+            use_build_command=use_build_command,
         ),
     ]
 
@@ -227,7 +345,9 @@ def load_assets_from_dbt_manifest(
     ] = None,
     io_manager_key: Optional[str] = None,
     selected_unique_ids: Optional[AbstractSet[str]] = None,
+    select: Optional[str] = None,
     node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
+    use_build_command: bool = False,
 ) -> Sequence[AssetsDefinition]:
     """
     Loads a set of dbt models, described in a manifest.json, into Dagster assets.
@@ -249,24 +369,25 @@ def load_assets_from_dbt_manifest(
         node_info_to_asset_key: (Mapping[str, Any] -> AssetKey): A function that takes a dictionary
             of dbt node info and returns the AssetKey that you want to represent that node. By
             default, the asset key will simply be the name of the dbt model.
+        use_build_command: (bool): Flag indicating if you want to use `dbt build` as the core computation
+            for this asset, rather than `dbt run`.
     """
     check.dict_param(manifest_json, "manifest_json", key_type=str)
     dbt_nodes = {**manifest_json["nodes"], **manifest_json["sources"]}
 
-    def _unique_id_to_selector(uid):
-        # take the fully-qualified node name and use it to select the model
-        return ".".join(dbt_nodes[uid]["fqn"])
+    if select is None:
+        if selected_unique_ids:
+            # generate selection string from unique ids
+            select = " ".join(".".join(dbt_nodes[uid]["fqn"]) for uid in selected_unique_ids)
+        else:
+            # if no selection specified, default to "*"
+            select = "*"
+            selected_unique_ids = manifest_json["nodes"].keys()
+
+    if selected_unique_ids is None:
+        # must resolve the selection string using the existing manifest.json data (hacky)
+        selected_unique_ids = _select_unique_ids_from_manifest_json(manifest_json, select)
 
-    select = (
-        "*"
-        if selected_unique_ids is None
-        else " ".join(_unique_id_to_selector(uid) for uid in selected_unique_ids)
-    )
-    selected_unique_ids = selected_unique_ids or set(
-        unique_id
-        for unique_id, node_info in dbt_nodes.items()
-        if node_info["resource_type"] == "model"
-    )
     return [
         _dbt_nodes_to_assets(
             dbt_nodes,
@@ -275,5 +396,6 @@ def load_assets_from_dbt_manifest(
             select=select,
             selected_unique_ids=selected_unique_ids,
             node_info_to_asset_key=node_info_to_asset_key,
+            use_build_command=use_build_command,
         )
     ]
