@@ -1,9 +1,9 @@
 import json
 import os
 import subprocess
-from typing import Any, Dict
+from typing import Any, Dict, Optional
 
-from dagster import check
+import dagster._check as check
 from dagster.core.utils import coerce_valid_log_level
 
 from ..errors import (
@@ -23,6 +23,7 @@ def execute_cli(
     warn_error: bool,
     ignore_handled_error: bool,
     target_path: str,
+    docs_url: Optional[str] = None,
 ) -> DbtCliOutput:
     """Executes a command on the dbt CLI in a subprocess."""
     check.str_param(executable, "executable")
@@ -111,6 +112,7 @@ def execute_cli(
         raw_output=raw_output,
         logs=logs,
         result=run_results,
+        docs_url=docs_url,
     )
 
 
@@ -122,3 +124,13 @@ def parse_run_results(path: str, target_path: str = DEFAULT_DBT_TARGET_PATH) ->
             return json.load(file)
     except FileNotFoundError:
         raise DagsterDbtCliOutputsNotFoundError(path=run_results_path)
+
+
+def parse_manifest(path: str, target_path: str = DEFAULT_DBT_TARGET_PATH) -> Dict[str, Any]:
+    """Parses the `target/manifest.json` artifact that is produced by a dbt process."""
+    manifest_path = os.path.join(path, target_path, "manifest.json")
+    try:
+        with open(manifest_path, encoding="utf8") as file:
+            return json.load(file)
+    except FileNotFoundError:
+        raise DagsterDbtCliOutputsNotFoundError(path=manifest_path)
